@@ -1,6 +1,10 @@
 type 'a cell =
   | Nil
   | Cons of { content: 'a; mutable next: 'a cell }
+[@@warning "-37"]
+(* Disable warning 37 (Unused constructor):
+   Cons is never used to build values,
+   but it is used implicitly in [of_abstr] *)
 
 type 'a t = {
   mutable length: int;
